use anyhow::Result;
use rosc::{OscBundle, OscMessage, OscPacket, encoder::encode};
use tokio::{net::UdpSocket, sync::mpsc::Receiver, task::JoinSet};

use crate::Args;

pub async fn setup_clients(args: Args, rx: Receiver<OscPacket>) -> Result<()> {
    println!("Starting client");

    let mut set = JoinSet::new();
    set.spawn(client(args.clone(), rx));

    while let Some(res) = set.join_next().await {
        res??;
    }

    Ok(())
}

async fn client(args: Args, rx: Receiver<OscPacket>) -> Result<()> {
    let mut rx = rx;
    let clients = args.clients.clone();
    let sock = UdpSocket::bind(format!("0.0.0.0:{}", args.client_send_port)).await?;

    loop {
        let packet = rx.recv().await;
        match packet {
            Some(packet) => {
                if let Some(packet) = translate_packet(packet) {
                    let packet = encode(&packet)?;
                    for client in &clients {
                        sock.send_to(&packet, client).await?;
                    }
                }
            }
            None => {
                println!("No more packets");
                return Ok(());
            }
        };
    }
}

fn translate_packet(packet: OscPacket) -> Option<OscPacket> {
    match packet {
        OscPacket::Message(ref msg) => match msg.addr.as_str() {
            "/Carla/param" => Some(packet),
            "/Carla/cb" => Some(OscPacket::Message(OscMessage {
                addr: String::from("/Carla/param"),
                args: vec![
                    msg.args[1].clone(),
                    msg.args[2].clone(),
                    msg.args[5].clone(),
                ],
            })),
            _ => None,
        },
        OscPacket::Bundle(OscBundle { timetag, content }) => {
            let content = content.into_iter().filter_map(translate_packet).collect();
            Some(OscPacket::Bundle(OscBundle { timetag, content }))
        }
    }
}
