use tokio::{sync::mpsc, task::JoinSet};

mod carla;
mod client;
mod server;

#[derive(Debug, Clone)]
struct Args {
    bridge_host: String,
    carla_host: String,
    clients: Vec<String>,
    carla_recv_port: u16,
    carla_send_port: u16,
    client_recv_port: u16,
    client_send_port: u16,
}

#[tokio::main]
async fn main() {
    let args = Args {
        bridge_host: String::from("10.20.60.254"),
        carla_host: String::from("10.20.60.251:22752"),
        clients: vec![String::from("10.20.60.251:8080")],
        carla_recv_port: 10400,
        carla_send_port: 10401,
        client_recv_port: 10402,
        client_send_port: 10403,
    };

    let (from_carla_tx, from_carla_rx) = mpsc::channel(1024);
    let (to_carla_tx, to_carla_rx) = mpsc::channel(1024);

    let mut set = JoinSet::new();
    set.spawn(carla::setup_carla(args.clone(), from_carla_tx));
    set.spawn(client::setup_clients(args.clone(), from_carla_rx));
    set.spawn(server::setup_server(args.clone(), to_carla_tx));
    set.spawn(carla::setup_forwarder(args, to_carla_rx));

    while let Some(res) = set.join_next().await {
        res.unwrap().unwrap();
    }
}
